# Milestone graphs

These tools generate *milestone graphs from* the input data.

The input are command-line logs in the folder `data_file_wrangler_tagged` or `data_locust_3302_tagged`.
* `data_file_wrangler_tagged`: Annotated data from 26 students participating in the File Wrangler exercise in the [EDURange](http://www.edurange.org/) platform.
* `data_locust_3302_tagged`: Annotated data from 20 students participating in the Locust exercise in the [KYPO Cyber Range Platform](https://crp.kypo.muni.cz/). These data were collected using a logging toolset referenced in the paper [Toolset for Collecting Shell Commands and Its Application in Hands-on Cybersecurity Training](https://www.muni.cz/en/research/publications/1783801).

The output are the graphs in the folder `graphs_file_wrangler` or `graphs_locust_3302`.

## Milestone Graph

The top level of nodes (white) are template nodes associated with individual milestones a student is expected to attempt.

Descending from each template node, each rectangular node represents student activity that was identified to be related to the template (entry nodes). 

At the bottom of each milestone chain is an elliptical summary node.

The colors of each of these entry nodes correspond to:
- No attempts recognized (red)
- Attempted but not achieved (yellow)
- Milestone successfully achieved (green)


![Milestone Graph](./example.svg)

Data located in `data/edurange_file_wrangler` are from Lewis and Clark CS293 Spring 2021.

## Description of milestones

The milestones for students are specified in terms of regular expressions.

Each milestone has three parts: Node, Terminal Input, Terminal Output.

Each of these is a regular expression, the Node is the context e.g. which directory and container. In milestones-wrangler.txt, this node field is always an asterisk (*) becasue this scenario only includes a single node. 

## Graph generation scripts

To run the scripts, Linux package `graphviz` must be installed:

```commandline
sudo apt-get install graphviz
```

The generation of student milestone graphs happens in two stages:

First, the raw data collected from the scenario must be tagged, and then graphs are generated from the tagged data.

Both of these operations are automated through running `make_graphs.sh` as follows:

```
./make_graphs.sh <input_dir> <milestone_file> <tag_dir> <output_dir>
```

For example with the data provided in this repository:

```
./make_graphs.sh ../../data/edurange_file_wrangler/ ../milestones-wrangler.txt ../edurange_tagged_student_data ../edurange_data_file_wrangler/
```
