# Trainee graphs

These tools generate *trainee graphs from* the input data.

The input are command-line logs in the folder `data_file_wrangler` or `data_locust_3302`.
* `data_file_wrangler`: Data from 26 students participating in the File Wrangler exercise in the [EDURange](http://www.edurange.org/) platform.
* `data_locust_3302`: Data from 20 students participating in the Locust exercise in the [KYPO Cyber Range Platform](https://crp.kypo.muni.cz/). These data were collected using a logging toolset referenced in the paper [Toolset for Collecting Shell Commands and Its Application in Hands-on Cybersecurity Training](https://www.muni.cz/en/research/publications/1783801).

The output are the graphs in the folder `graphs_file_wrangler` or `graphs_locust_3302`.

## Graph generation scripts

To run the scripts, Linux package `graphviz` must be installed:

```commandline
sudo apt-get install graphviz
```

### create_trainee_graph.py

For creating a trainee graph for each student.

Usage:

```commandline
create_trainee_graph.py reference_graph command_logs_folder [-f FORMAT]
```
Arguments:

- `reference_graph` - path to the reference graph (`.gv` file)
- `command_logs_folder` - folder with the command logs of trainees (`.json` files)
- `-f FORMAT` (optional) - file format of the drawn graph (PDF by default), [allowed formats](https://www.graphviz.org/doc/info/output.html)

Created trainee graphs will be saved in a folder called `trainee_graphs` located in a folder specified by the argument `command_logs_folder`. If the folder`trainee_graphs` doesn't exist, it will be created.

Example with our data:

```commandline
create_trainee_graph.py ../reference_graphs/reference_graph_kypo.gv ../data_locust_3302/
```

### create_summary_graph.py

For creating a summary graph from multiple trainee graphs.

Usage:

```commandline
create_summary_graph.py reference_graph trainee_graphs_folder [-f FORMAT] 
```

Arguments:

- `reference_graph` - path to the reference graph (`.gv` file)
- `trainee_graphs_folder` - folder with the trainee graphs (`.gv` files)
- `-f FORMAT` (optional) - file format of the drawn graph (PDF by default), [allowed formats](https://www.graphviz.org/doc/info/output.html)

The created summary graph will be created in a folder called `summary_graph` located in a folder specified by the argument `trainee_graphs_folder`. If the folder `summary_graph` doesn't exist, it will be created.

Example with our data:

```commandline
create_summary_graph.py ../reference_graphs/reference_graph_kypo.gv ../data_locust_3302/
```

Data located in `data/kypo_locust_3302` are from a Summer School in July 2020.

## Succession of nodes

For each node, its neighboring nodes are visited in the following order:
* error nodes (red) first,
* then possible nodes (orange),
* then success nodes (green)

Visual example:

![Succession of nodes](example.png?raw=True "Succession of nodes - visualised")
